

#ifndef  __MIDIPLAYER_H__
#define  __MIDIPLAYER_H__


#include <string>
#include <vector>
#include "types.H"


extern "C++" {
	using namespace std;
	namespace algarrobo {

		class TMidiPlayer {
			public:
				virtual void sendMidiEvent(u8 *data, u8 size) = 0;
		};

		class TDummyMidiPlayer : public TMidiPlayer {
			public:
				void sendMidiEvent(u8 *data, u8 size);
		};

		class TMidiTrack {
			protected:
				u8 *data;
				TMidiPlayer *player;
				u8 *pointer;
				u32 deltaCounter;
				u8 runningStatus;
				bool playing;
				u32 getNextValue();
				u32 getVarLen();
				u8 numDataBytes(u8 status);
			public:
				TMidiTrack(u8 *p, TMidiPlayer &player);
				void play();
				void stop();
				s32 tick();
		};

		class TMidiFileParser {
			protected:
				u8 *midiFile;
				u16 format;
				u16 numTracks;
				u16 division;
				u32 deltaTimeIntervalMicroseconds;
				u32 currentTempo;                      // microseconds per quarter note
				vector<TMidiTrack *> tracks;
				TMidiPlayer *player;
				bool playing;
				void updateDeltaTimeInterval();
				void freeTracks();
			public:
				TMidiFileParser(TMidiPlayer &player);
				~TMidiFileParser();
				void load(u8 *data);
				void play();
				void stop();
				u32 getDeltaTimeIntervalMicroseconds();
				bool tick(bool &deltaTimeIntervalChanged);
		};

	}
}

#endif  // __MIDIPLAYER_H__

