

#include <string>
#include <vector>
#include <iostream>
#include <string.h>
#include <stdio.h>
#include "types.H"
#include "midiplayer.H"


using namespace std;
using namespace algarrobo;


#if  __BYTE_ORDER == __LITTLE_ENDIAN

#define  FROM_BIG_ENDIAN_16(v)  ((((v) >> 8) & 0x00FF) | (((v) << 8) & 0xFF00))
#define  FROM_BIG_ENDIAN_32(v)  ((((v) >> 24) & 0x000000FF) | (((v) >> 8) & 0x0000FF00) | (((v) << 8) & 0x00FF0000) | (((v) << 24) & 0xFF000000))

#else

#define  FROM_BIG_ENDIAN_16(v)  (v)
#define  FROM_BIG_ENDIAN_32(v)  (v)

#endif


//------------------
// TDummyMidiPlayer
//------------------


void TDummyMidiPlayer::sendMidiEvent(u8 *data, u8 size) {
	for (u8 i = 0; i < size; i++)
		printf("%02X ", (unsigned int) data[i]);
	cout << endl;
}


//------------
// TMidiTrack
//------------


TMidiTrack::TMidiTrack(u8 *p, TMidiPlayer &player) {
	this->data = p;
	this->pointer = this->data;
	this->player = &player;
	this->deltaCounter = 0;
	this->runningStatus = 0;
	this->playing = false;
}


void TMidiTrack::play() {
	this->deltaCounter = 0;
	this->runningStatus = 0;
	this->playing = true;
	this->pointer = this->data;
}


void TMidiTrack::stop() {
	this->playing = false;
}


u32 TMidiTrack::getNextValue() {
	u8 ret = *this->pointer;
	this->pointer++;
	return (u32) ret;
}


u32 TMidiTrack::getVarLen() {
	u32 value;
	u8 c;
	if ((value = this->getNextValue()) & 0x80) {
		value &= 0x7f;
		do {
			value = (value << 7) + ((c = this->getNextValue()) & 0x7f);
		} while (c & 0x80);
	}
	return value;
}


u8 TMidiTrack::numDataBytes(u8 status) {
	status >>= 4;
	if ((status == 0x08) || (status == 0x09) || (status == 0x0A) || (status == 0x0B) || (status == 0x0E))
		return 2;
	return 1;
}


s32 TMidiTrack::tick() {
	if (!this->playing)
		return -2;
	u32 newTempo = -1;
	if (this->deltaCounter > 0) {
		this->deltaCounter--;
		return newTempo;
	}
	do {
		this->deltaCounter = this->getVarLen();
		cout << "delta=" << this->deltaCounter << flush;
		u32 firstEventByte = (u32) *this->pointer;
		cout << "  firstEventByte=" << firstEventByte << endl << flush;
		if (firstEventByte == 0xFF) {
			// meta-event
			this->pointer++;
			u8 type = *(this->pointer++);
			u32 len = this->getVarLen();
			if (type == 0x51) {
				// "set tempo" meta event
				newTempo = ((u32) *(this->pointer++)) << 16;
				newTempo |= ((u32) *(this->pointer++)) << 8;
				newTempo |= ((u32) *(this->pointer++));
				cout << "set tempo to " << newTempo << endl;
			}
			else if (type == 0x2F) {
				// "end of track" meta event
				this->playing = false;
				cout << "end of track" << endl;
				break;
			}
			else {
				// ignore any other meta event
				this->pointer += len;
			}
		}
		else if ((firstEventByte == 0xF0) || (firstEventByte == 0xF7)) {
			// sysex event
			this->pointer++;
			u32 len = this->getVarLen();
			this->pointer += len;    // discard sysex...
		}
		else {
			// midi event
			if (firstEventByte > 0x7F) {
				// new status
				this->runningStatus = (u8) firstEventByte;
			}
			u8 bytes = this->numDataBytes(this->runningStatus);
			if (firstEventByte > 0x7F) {
				// new status
				bytes++;
			}
			this->player->sendMidiEvent(this->pointer, bytes);
			this->pointer += bytes;
		}
	} while (this->deltaCounter == 0);
	return newTempo;
}


//-----------------
// TMidiFileParser
//-----------------


TMidiFileParser::TMidiFileParser(TMidiPlayer &player) {
	this->player = &player;
	this->playing = false;
	this->numTracks = 0;
	this->midiFile = NULL;
}


void TMidiFileParser::load(u8 *data) {
	this->freeTracks();
	char aux[5];
	aux[4] = 0;
	this->midiFile = data;
	if (strncmp((const char *) this->midiFile, "MThd", 4) != 0)
		throw bad_alloc();
	strncpy(aux, (const char *) this->midiFile, 4);
	cout << "'" << aux << "'" << endl;
	this->format = FROM_BIG_ENDIAN_16(*((u16 *) (&this->midiFile[8])));
	if (this->format > 1)
		throw bad_alloc();
	this->numTracks = FROM_BIG_ENDIAN_16(*((u16 *) (&this->midiFile[10])));
	this->division = FROM_BIG_ENDIAN_16(*((u16 *) (&this->midiFile[12])));
	cout << "format=" << this->format << " numTracks=" << this->numTracks << " division=" << this->division << endl;
	s32 trackOffset = 14;
	for (s32 i = 0; i < this->numTracks; i++) {
		s32 trackLength = FROM_BIG_ENDIAN_32(*((s32 *) &this->midiFile[trackOffset + 4]));
		strncpy(aux, (const char *) &this->midiFile[trackOffset], 4);
		cout << "\t'" << aux << "'  (" << trackLength << " bytes)" << endl << flush;
		this->tracks.push_back(new TMidiTrack(&this->midiFile[trackOffset + 8], *this->player));
		trackOffset += 8 + trackLength;
	}
	this->currentTempo = 500000;   // default tempo = 120bpm
	this->updateDeltaTimeInterval();
}


void TMidiFileParser::updateDeltaTimeInterval() {
	this->deltaTimeIntervalMicroseconds = this->currentTempo / ((u32) this->division);
	cout << "deltaTimeIntervalMicroseconds=" << this->deltaTimeIntervalMicroseconds << endl;
}


u32 TMidiFileParser::getDeltaTimeIntervalMicroseconds() {
	return this->deltaTimeIntervalMicroseconds;
}


void TMidiFileParser::play() {
	this->playing = true;
	for (s32 i = 0; i < this->numTracks; i++)
		this->tracks[i]->play();
}


void TMidiFileParser::stop() {
	this->playing = false;
	for (s32 i = 0; i < this->numTracks; i++)
		this->tracks[i]->stop();
}


bool TMidiFileParser::tick(bool &deltaTimeIntervalChanged) {
	if (!this->playing)
		return false;
	s32 newTempo = -1;
	s32 numTracksPlaying = 0;
	for (s32 i = 0; i < this->numTracks; i++) {
		s32 aux = this->tracks[i]->tick();
		if (aux >= 0)
			newTempo = aux;
		if (aux != -2)   // -2 means that the track is not playing any data
			numTracksPlaying++;
	}
	if (newTempo > 0) {
		this->currentTempo = newTempo;
		this->updateDeltaTimeInterval();
		deltaTimeIntervalChanged = true;
	}
	else
		deltaTimeIntervalChanged = false;
	return (numTracksPlaying > 0);
}


void TMidiFileParser::freeTracks() {
	for (int i = 0; i < this->numTracks; i++)
		delete this->tracks[i];
	this->tracks.clear();
}


TMidiFileParser::~TMidiFileParser() {
	this->freeTracks();
}

