

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include "midifileparser.h"

// unix specific includes
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


void sendMidiEventCallback(uint8_t *data, uint8_t dataLength) {
	uint8_t i;
	for (i = 0; i < dataLength; i++)
		printf("%02X ", (unsigned int) data[i]);
	printf("\n");
}


int main(int argc, char **argv) {
	uint8_t *midiData;
	midiFileParser parser;
	struct stat statBuf;
	int fd;
	uint8_t deltaTimeIntervalChanged;
	// load midi file into memory
	if (argc < 2) {
		fprintf(stderr, "Syntax: %s midifile\n", argv[0]);
		return 1;
	}
	fd = open(argv[1], O_RDONLY);
	if (fd <= 0) {
		fprintf(stderr, "Could not load '%s'\n", argv[1]);
		return 1;
	}
	fstat(fd, &statBuf);
	printf("%d bytes\n", (int) statBuf.st_size);
	midiData = (uint8_t *) malloc(statBuf.st_size);
	read(fd, midiData, statBuf.st_size);
	close(fd);
	// play midi
	midiFileParserInit(&parser, sendMidiEventCallback);
	midiFileParserLoad(&parser, midiData);
	midiFileParserPlay(&parser);
	deltaTimeIntervalChanged = 0;
	while (midiFileParserTick(&parser, &deltaTimeIntervalChanged)) {
		uint32_t delta = midiFileParserGetDeltaTimeIntervalMicroseconds(&parser);
		// cout << "delta: " << delta << endl;
		usleep(delta);
	}
	midiFileParserStop(&parser);
	// free memory
	midiFileParserRelease(&parser);
	return 0;
}
