

#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include "midifileparser.h"


#if  __BYTE_ORDER == __LITTLE_ENDIAN

#define  FROM_BIG_ENDIAN_16(v)  ((((v) >> 8) & 0x00FF) | (((v) << 8) & 0xFF00))
#define  FROM_BIG_ENDIAN_32(v)  ((((v) >> 24) & 0x000000FF) | (((v) >> 8) & 0x0000FF00) | (((v) << 8) & 0x00FF0000) | (((v) << 24) & 0xFF000000))

#else

#define  FROM_BIG_ENDIAN_16(v)  (v)
#define  FROM_BIG_ENDIAN_32(v)  (v)

#endif


void midiFileParserInit(midiFileParser *t, void (*sendMidiEventCallback)(uint8_t *, uint8_t)) {
	t->sendMidiEventCallback = sendMidiEventCallback;
	t->playing = 0;
	t->numTracks = 0;
	t->tracks = NULL;
	t->midiFile = NULL;
}


uint8_t midiFileParserLoad(midiFileParser *t, uint8_t *data) {
	midiTrack *currentTrack;
	char aux[5];
	int32_t trackOffset;
	uint16_t i;
	midiFileParserFreeTracks(t);
	aux[4] = 0;
	t->midiFile = data;
	if (strncmp((const char *) t->midiFile, "MThd", 4) != 0)
		return 0;
	strncpy(aux, (const char *) t->midiFile, 4);
	printf("'%s'\n", aux);
	t->format = FROM_BIG_ENDIAN_16(*((uint16_t *) (&t->midiFile[8])));
	if (t->format > 1)
		return 0;
	t->numTracks = FROM_BIG_ENDIAN_16(*((uint16_t *) (&t->midiFile[10])));
	t->tracks = (midiTrack *) malloc(t->numTracks * sizeof(midiTrack));
	if (t->tracks == NULL)
		return 0;
	currentTrack = t->tracks;
	t->division = FROM_BIG_ENDIAN_16(*((uint16_t *) (&t->midiFile[12])));
	printf("format=%d numTracks=%d division=%d\n", (int) t->format, (int) t->numTracks, (int) t->division);
	trackOffset = 14;
	for (i = 0; i < t->numTracks; i++, currentTrack++) {
		int32_t trackLength = FROM_BIG_ENDIAN_32(*((int32_t *) &t->midiFile[trackOffset + 4]));
		strncpy(aux, (const char *) &t->midiFile[trackOffset], 4);
		printf("\t'%s'  (%d bytes)", aux, (int) trackLength);
		midiTrackInit(currentTrack, &t->midiFile[trackOffset + 8], t->sendMidiEventCallback);
		trackOffset += 8 + trackLength;
	}
	t->currentTempo = 500000;   // default tempo = 120bpm
	midiFileParserUpdateDeltaTimeInterval(t);
	return 1;
}


void midiFileParserUpdateDeltaTimeInterval(midiFileParser *t) {
	t->deltaTimeIntervalMicroseconds = t->currentTempo / ((uint32_t) t->division);
	printf("deltaTimeIntervalMicroseconds=%d\n", t->deltaTimeIntervalMicroseconds);
}


uint32_t midiFileParserGetDeltaTimeIntervalMicroseconds(midiFileParser *t) {
	return t->deltaTimeIntervalMicroseconds;
}


void midiFileParserPlay(midiFileParser *t) {
	int16_t i;
	midiTrack *currentTrack = t->tracks;
	t->playing = 1;
	for (i = 0; i < t->numTracks; i++, currentTrack++)
		midiTrackPlay(currentTrack);
}


void midiFileParserStop(midiFileParser *t) {
	int16_t i;
	t->playing = 0;
	midiTrack *currentTrack = t->tracks;
	for (i = 0; i < t->numTracks; i++, currentTrack++)
		midiTrackStop(currentTrack);
}


uint8_t midiFileParserTick(midiFileParser *t, uint8_t *deltaTimeIntervalChanged) {
	int32_t newTempo = -1;
	int32_t numTracksPlaying = 0;
	uint16_t i;
	midiTrack *currentTrack;
	if (!t->playing)
		return 0;
	currentTrack = t->tracks;
	for (i = 0; i < t->numTracks; i++, currentTrack++) {
		int32_t aux = midiTrackTick(currentTrack);
		if (aux >= 0)
			newTempo = aux;
		if (aux != -2)   // -2 means that the track is not playing any data
			numTracksPlaying++;
	}
	if (newTempo > 0) {
		t->currentTempo = newTempo;
		midiFileParserUpdateDeltaTimeInterval(t);
		*deltaTimeIntervalChanged = 1;
	}
	else
		*deltaTimeIntervalChanged = 0;
	return (numTracksPlaying > 0) ? 1 : 0;
}


void midiFileParserFreeTracks(midiFileParser *t) {
	if (t->tracks != NULL) {
		free(t->tracks);
		t->tracks = NULL;
	}
}


void midiFileParserRelease(midiFileParser *t) {
	midiFileParserFreeTracks(t);
}

