

#include "miditrack.h"
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>


void midiTrackInit(midiTrack *t, uint8_t *p, void (*sendMidiEventCallback)(uint8_t *, uint8_t)) {
	t->data = p;
	t->pointer = t->data;
	t->sendMidiEventCallback = sendMidiEventCallback;
	t->deltaCounter = 0;
	t->runningStatus = 0;
	t->playing = 0;
}

void midiTrackPlay(midiTrack *t) {
	t->deltaCounter = 0;
	t->runningStatus = 0;
	t->playing = 1;
	t->pointer = t->data;
}

void midiTrackStop(midiTrack *t) {
	t->playing = 0;
}

uint32_t midiTrackGetNextValue(midiTrack *t) {
	uint8_t ret = *t->pointer;
	t->pointer++;
	return (uint32_t) ret;
}

uint32_t midiTrackGetVarLen(midiTrack *t) {
	uint32_t value;
	uint8_t c;
	if ((value = midiTrackGetNextValue(t)) & 0x80) {
		value &= 0x7f;
		do {
			value = (value << 7) + ((c = midiTrackGetNextValue(t)) & 0x7f);
		} while (c & 0x80);
	}
	return value;
}

uint8_t midiTrackNumDataBytes(uint8_t status) {
    status >>= 4;
    if ((status == 0x08) || (status == 0x09) || (status == 0x0A) || (status == 0x0B) || (status == 0x0E))
        return 2;
    return 1;
}

int32_t midiTrackTick(midiTrack *t) {
	if (!t->playing)
		return -2;
	uint32_t newTempo = -1;
	if (t->deltaCounter > 0) {
		t->deltaCounter--;
		return newTempo;
	}
	do {
		uint32_t firstEventByte;
		t->deltaCounter = midiTrackGetVarLen(t);
		printf("delta=%u", t->deltaCounter);
		firstEventByte = (uint32_t) *t->pointer;
		printf("  firstEventByte=%u\n", firstEventByte);
		if (firstEventByte == 0xFF) {
			// meta-event
			t->pointer++;
			uint8_t type = *(t->pointer++);
			uint32_t len = midiTrackGetVarLen(t);
			if (type == 0x51) {
				// "set tempo" meta event
				newTempo = ((uint32_t) *(t->pointer++)) << 16;
				newTempo |= ((uint32_t) *(t->pointer++)) << 8;
				newTempo |= ((uint32_t) *(t->pointer++));
				printf("set tempo to %d\n", newTempo);
			}
			else if (type == 0x2F) {
				// "end of track" meta event
				t->playing = 0;
				printf("end of track\n");
				break;
			}
			else {
				// ignore any other meta event
				t->pointer += len;
			}
		}
		else if ((firstEventByte == 0xF0) || (firstEventByte == 0xF7)) {
			// sysex event
			uint32_t len;
			t->pointer++;
			len = midiTrackGetVarLen(t);
			t->pointer += len;    // discard sysex...
		}
		else {
			// midi event
			uint8_t bytes;
			if (firstEventByte > 0x7F) {
				// new status
				t->runningStatus = (uint8_t) firstEventByte;
			}
			bytes = midiTrackNumDataBytes(t->runningStatus);
			if (firstEventByte > 0x7F) {
				// new status
				bytes++;
			}
			t->sendMidiEventCallback(t->pointer, bytes);
			t->pointer += bytes;
		}
	} while (t->deltaCounter == 0);
	return newTempo;
}

